@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 #include <stdio.h>
 #include <stdlib.h>
@@ -56,7 +56,8 @@ typedef enum {
   UPDATE_XY_COLOR = 1 << 2,
   UPDATE_Z_COLOR = 1 << 3,
   UPDATE_NEW_COLOR = 1 << 4,
-  UPDATE_ORIG_COLOR = 1 << 5
+  UPDATE_ORIG_COLOR = 1 << 5,
+  UPDATE_CALLER = 1 << 6
 } ColorSelectUpdateType;
 
 typedef struct _ColorSelectFill ColorSelectFill;
@@ -72,6 +73,7 @@ struct _ColorSelectFill {
 };
 
 static void color_select_update (ColorSelectP, ColorSelectUpdateType);
+static void color_select_update_caller (ColorSelectP);
 static void color_select_update_values (ColorSelectP);
 static void color_select_update_rgb_values (ColorSelectP);
 static void color_select_update_hsv_values (ColorSelectP);
@@ -82,6 +84,7 @@ static void color_select_update_colors (ColorSelectP, int);
 
 static void color_select_ok_callback (GtkWidget *, gpointer);
 static void color_select_cancel_callback (GtkWidget *, gpointer);
+static gint color_select_delete_callback (GtkWidget *, GdkEvent *, gpointer);
 static gint color_select_xy_expose (GtkWidget *, GdkEventExpose *, ColorSelectP);
 static gint color_select_xy_events (GtkWidget *, GdkEvent *, ColorSelectP);
 static gint color_select_z_expose (GtkWidget *, GdkEventExpose *, ColorSelectP);
@@ -136,7 +139,8 @@ color_select_new (int                  r,
 		  int                  g,
 		  int                  b,
 		  ColorSelectCallback  callback,
-		  void                *client_data)
+		  void                *client_data,
+		  int                  wants_updates)
 {
   /*  static char *toggle_titles[6] = { "Hue", "Saturation", "Value", "Red", "Green", "Blue" }; */
   static char *toggle_titles[6] = { "H", "S", "V", "R", "G", "B" };
@@ -164,6 +168,7 @@ color_select_new (int                  r,
   csp->z_color_fill = HUE;
   csp->xy_color_fill = SATURATION_VALUE;
   csp->gc = NULL;
+  csp->wants_updates = wants_updates;
 
   csp->values[RED] = csp->orig_values[0] = r;
   csp->values[GREEN] = csp->orig_values[1] = g;
@@ -172,10 +177,15 @@ color_select_new (int                  r,
   color_select_update_pos (csp);
 
   csp->shell = gtk_dialog_new ();
+  gtk_window_set_wmclass (GTK_WINDOW (csp->shell), "color_selection", "Gimp");
   gtk_window_set_title (GTK_WINDOW (csp->shell), "Color Selection");
   gtk_window_set_policy (GTK_WINDOW (csp->shell), FALSE, FALSE, FALSE);
   gtk_widget_set_uposition (csp->shell, color_select_x, color_select_y);
 
+  /*  handle the wm close signal */
+  gtk_signal_connect (GTK_OBJECT (csp->shell), "delete_event",
+		      (GtkSignalFunc) color_select_delete_callback, csp);
+  
   main_vbox = gtk_vbox_new (FALSE, 2);
   gtk_container_border_width (GTK_CONTAINER (main_vbox), 2);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (csp->shell)->vbox), main_vbox, TRUE, TRUE, 0);
@@ -306,6 +316,16 @@ color_select_new (int                  r,
   /*  The action area  */
   action_items[0].user_data = csp;
   action_items[1].user_data = csp;
+  if (csp->wants_updates)
+    {
+      action_items[0].label = "Close";
+      action_items[1].label = "Revert to Old Color";
+    }
+  else
+    {
+      action_items[0].label = "OK";
+      action_items[1].label = "Cancel";
+    }
   build_action_area (GTK_DIALOG (csp->shell), action_items, 2, 0);
 
   color_select_image_fill (csp->z_color, csp->z_color_fill, csp->values);
@@ -410,6 +430,22 @@ color_select_update (ColorSelectP          csp,
 
       if (update & UPDATE_ORIG_COLOR)
 	color_select_update_colors (csp, 1);
+
+      /*if (update & UPDATE_CALLER)*/
+      color_select_update_caller (csp);
+    }
+}
+
+static void
+color_select_update_caller (ColorSelectP csp)
+{
+  if (csp && csp->wants_updates && csp->callback)
+    {
+      (* csp->callback) (csp->values[RED],
+			 csp->values[GREEN],
+			 csp->values[BLUE],
+			 COLOR_SELECT_UPDATE,
+			 csp->client_data);
     }
 }
 
@@ -584,6 +620,7 @@ color_select_update_hsv_values (ColorSelectP csp)
 	h = 0;
       else
 	{
+	  h = 0;
 	  delta = max - min;
 	  if (r == max)
 	    h = (g - b) / (float) delta;
@@ -739,10 +776,22 @@ color_select_ok_callback (GtkWidget *w,
 	(* csp->callback) (csp->values[RED],
 			   csp->values[GREEN],
 			   csp->values[BLUE],
-			   0, csp->client_data);
+			   COLOR_SELECT_OK,
+			   csp->client_data);
     }
 }
 
+static gint
+color_select_delete_callback (GtkWidget *w,
+			      GdkEvent  *e,
+			      gpointer   client_data)
+{
+  color_select_cancel_callback (w, client_data);
+
+  return TRUE;
+}
+  
+
 static void
 color_select_cancel_callback (GtkWidget *w,
 			      gpointer   client_data)
@@ -756,7 +805,8 @@ color_select_cancel_callback (GtkWidget *w,
 	(* csp->callback) (csp->orig_values[0],
 			   csp->orig_values[1],
 			   csp->orig_values[2],
-			   1, csp->client_data);
+			   COLOR_SELECT_CANCEL,
+			   csp->client_data);
     }
 }
 
@@ -1345,6 +1395,8 @@ color_select_update_hue (ColorSelectFill *csf)
   h /= 60;
   f = (h - (int) h) * 255;
 
+  r = g = b = 0;
+
   switch ((int) h)
     {
     case 0:
@@ -1545,7 +1597,7 @@ color_select_update_hue_saturation (ColorSelectFill *csf)
   p = csf->buffer;
 
   csf->y += 1;
-  h = (255 - csf->y) * 359 / csf->height;
+  h = 360 - (csf->y * 360 / csf->height);
 
   if (h < 0)
     h = 0;
@@ -1636,7 +1688,7 @@ color_select_update_hue_value (ColorSelectFill *csf)
   p = csf->buffer;
 
   csf->y += 1;
-  h = (255 - csf->y) * 359 / csf->height;
+  h = 360 - (csf->y * 360 / csf->height);
 
   if (h < 0)
     h = 0;
