@@ -13,11 +13,51 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+  /*
+   * TODO for Convert:
+   *
+   *   Use palette of another open INDEXED image
+   *
+   *   Different dither types
+   *
+   *   Alpha dithering
+   */
+
+/*
+ * 98/04/13 - avoid a division by zero when converting an empty gray-scale
+ *  image (who would like to do such a thing anyway??)  [Sven ] 
+ *
+ * 98/03/23 - fixed a longstanding fencepost - hopefully the *right*
+ *  way, *again*.  (anyone ELSE want a go?  okay, just kidding... :))
+ *  [Adam]
+ *
+ * 97/11/14 - added a proper pdb interface and support for dithering
+ *  to custom palettes (based on a patch by Eric Hernes) [Yosh]
+ *
+ * 97/11/04 - fixed the accidental use of the colour-counting case
+ *  when palette_type is WEB or MONO. [Adam]
+ *
+ * 97/10/25 - colour-counting implemented (could use some hashing, but
+ *  performance actually seems okay) - now RGB->INDEXED conversion isn't
+ *  destructive if it doesn't have to be. [Adam]
+ *
+ * 97/10/14 - fixed divide-by-zero when converting a completely transparent
+ *  RGB image to indexed. [Adam]
+ *
+ * 97/07/01 - started todo/revision log.  Put code back in to
+ *  eliminate full-alpha pixels from RGB histogram.
+ *  [Adam D. Moss - adam@gimp.org]
+ */
+
+
+
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
+
 #include "appenv.h"
 #include "actionarea.h"
 #include "convert.h"
@@ -28,18 +68,25 @@
 #include "indexed_palette.h"
 #include "interface.h"
 #include "undo.h"
+#include "palette.h"
 
+#include "layer_pvt.h"			/* ick. */
+#include "drawable_pvt.h"		/* ick ick. */
+#include "tile_manager_pvt.h"		/* ick ick ick. */
 
 #define MAXNUMCOLORS 256
 
 #define NODITHER 0
 #define FSDITHER 1
+#define NODESTRUCTDITHER 2
 
 /* adam's extra palette stuff */
 #define MAKE_PALETTE 0
 #define REUSE_PALETTE 1
 #define WEB_PALETTE 2
 #define MONO_PALETTE 3
+#define CUSTOM_PALETTE 4
+
 #define PRECISION_R 6
 #define PRECISION_G 6
 #define PRECISION_B 5
@@ -151,21 +198,25 @@ typedef struct
   GtkWidget *  shell;
   void *       gimage_ptr;
 
+  int          gimage_ID;
   int          dither;
   int          num_cols;
   int          palette;
   int          makepal_flag;
   int          webpal_flag;
+  int          custompal_flag;
   int          monopal_flag;
   int          reusepal_flag;
 } IndexedDialog;
 
-static Argument * convert_rgb_invoker       (Argument *);
-static Argument * convert_grayscale_invoker (Argument *);
-static Argument * convert_indexed_invoker   (Argument *);
+static Argument * convert_rgb_invoker               (Argument *);
+static Argument * convert_grayscale_invoker         (Argument *);
+static Argument * convert_indexed_invoker           (Argument *);
+static Argument * convert_indexed_palette_invoker   (Argument *);
 
 static void indexed_ok_callback     (GtkWidget *, gpointer);
 static void indexed_cancel_callback (GtkWidget *, gpointer);
+static gint indexed_delete_callback (GtkWidget *, GdkEvent *, gpointer);
 static void indexed_num_cols_update (GtkWidget *, gpointer);
 static void indexed_radio_update    (GtkWidget *, gpointer);
 static void indexed_dither_update   (GtkWidget *, gpointer);
@@ -177,10 +228,18 @@ static void grayscale_converter (Layer *, TileManager *, int);
 static void zero_histogram_gray     (Histogram);
 static void zero_histogram_rgb      (Histogram);
 static void generate_histogram_gray (Histogram, Layer *);
-static void generate_histogram_rgb  (Histogram, Layer *);
+static void generate_histogram_rgb  (Histogram, Layer *, int col_limit);
 
 static QuantizeObj* initialize_median_cut (int, int, int, int);
 
+static unsigned char found_cols[MAXNUMCOLORS][3];
+static int num_found_cols;
+static gboolean needs_quantize;
+
+static GtkWidget *build_palette_menu(int *default_palette);
+static void palette_entries_callback(GtkWidget *w, gpointer client_data);
+static PaletteEntriesP theCustomPalette = NULL;
+
 void
 convert_to_rgb (void *gimage_ptr)
 {
@@ -225,6 +284,7 @@ convert_to_indexed (void *gimage_ptr)
   gimage = (GImage *) gimage_ptr;
   dialog = (IndexedDialog *) g_malloc (sizeof (IndexedDialog));
   dialog->gimage_ptr = gimage_ptr;
+  dialog->gimage_ID = gimage->ID;
   dialog->num_cols = 256;
   dialog->dither = TRUE;
 
@@ -243,17 +303,21 @@ convert_to_indexed (void *gimage_ptr)
       dialog->num_cols = 255;
       if (!shown_message_already)
 	{
-	  message_box ("Note:  You are attempting to convert an image\nwith alpha/layers.  It is recommended that you quantize\nto no more than 255 colors if you wish to make\na transparent or animated GIF from it.\n\nYou won't get this message again\nuntil the next time you run GIMP.\nHave a nice day!",
-		       NULL, NULL);
+	  g_message ("Note:  You are attempting to convert an image\nwith alpha/layers.  It is recommended that you quantize\nto no more than 255 colors if you wish to make\na transparent or animated GIF from it.\n\nYou won't get this message again\nuntil the next time you run GIMP.\nHave a nice day!");
 	  shown_message_already = True;
 	}
     }
   dialog->makepal_flag = TRUE;
   dialog->webpal_flag = FALSE;
+  dialog->custompal_flag = FALSE;
   dialog->monopal_flag = FALSE;
   dialog->reusepal_flag = FALSE;
   dialog->shell = gtk_dialog_new ();
+  gtk_window_set_wmclass (GTK_WINDOW (dialog->shell), "indexed_color_conversion", "Gimp");
   gtk_window_set_title (GTK_WINDOW (dialog->shell), "Indexed Color Conversion");
+  gtk_signal_connect (GTK_OBJECT (dialog->shell), "delete_event",
+		      GTK_SIGNAL_FUNC (indexed_delete_callback),
+		      dialog);
 
   frame = gtk_frame_new ("Palette Options");
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
@@ -307,6 +371,9 @@ convert_to_indexed (void *gimage_ptr)
 
   if (gimage->base_type == RGB)
     {
+		GtkWidget *menu;
+		GtkWidget *palette_option_menu;
+		int default_palette;
       /*  'web palette'  */
       hbox = gtk_hbox_new (FALSE, 1);
       gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
@@ -320,6 +387,30 @@ convert_to_indexed (void *gimage_ptr)
       gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (toggle), dialog->webpal_flag);
       gtk_widget_show (toggle);
       gtk_widget_show (hbox);
+
+      menu = build_palette_menu(&default_palette);
+      if (menu) {
+          /* 'custom' palette from dialog */
+          hbox = gtk_hbox_new (FALSE, 1);
+          gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+          toggle = gtk_radio_button_new_with_label (group, "Use custom palette");
+          group = gtk_radio_button_group (GTK_RADIO_BUTTON (toggle));
+          gtk_box_pack_start (GTK_BOX (hbox), toggle, TRUE, TRUE, 0);
+          gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
+			      (GtkSignalFunc) indexed_radio_update,
+			      &(dialog->custompal_flag));
+          gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (toggle),
+				   dialog->custompal_flag);
+          gtk_widget_show (toggle);
+
+          palette_option_menu = gtk_option_menu_new();
+          gtk_option_menu_set_menu (GTK_OPTION_MENU(palette_option_menu), menu);
+          gtk_option_menu_set_history(GTK_OPTION_MENU(palette_option_menu),
+				      default_palette);
+          gtk_box_pack_start(GTK_BOX(hbox), palette_option_menu, TRUE, TRUE, 2);
+	  gtk_widget_show(palette_option_menu);
+          gtk_widget_show (hbox);
+      }
     }
 
   /*  'mono palette'  */
@@ -369,6 +460,56 @@ convert_to_indexed (void *gimage_ptr)
   gtk_widget_show (dialog->shell);
 }
 
+static GtkWidget *
+build_palette_menu(int *default_palette){
+  GtkWidget *menu;
+  GtkWidget *menu_item;
+  GSList *list;
+  PaletteEntriesP entries;
+  int i;
+
+
+  if(!palette_entries_list) {
+    /* fprintf(stderr, "no palette_entries_list, building...\n");*/
+     palette_init_palettes(FALSE);
+  }
+
+  list = palette_entries_list;
+
+  if (!list)
+    return NULL;
+
+  menu = gtk_menu_new();
+
+  for(i=0,list = palette_entries_list,*default_palette=-1;
+      list;
+      i++,list = g_slist_next (list))
+    {
+      entries = (PaletteEntriesP) list->data;
+      /*      fprintf(stderr, "(palette %s)\n", entries->filename);*/
+      menu_item = gtk_menu_item_new_with_label (entries->name);
+      gtk_signal_connect( GTK_OBJECT(menu_item), "activate",
+			  (GtkSignalFunc) palette_entries_callback,
+			  (gpointer)entries);
+      gtk_container_add(GTK_CONTAINER(menu), menu_item);
+      gtk_widget_show(menu_item);
+      if (theCustomPalette == entries) *default_palette = i;
+    }
+
+   /* default to first one */
+   if(*default_palette==-1) {
+     theCustomPalette = (PaletteEntriesP) palette_entries_list->data;
+     /* fprintf(stderr, "setting default to `%s'\n", theCustomPalette->name);*/
+     *default_palette = 0;
+   }
+   return menu;
+}
+
+static void
+palette_entries_callback(GtkWidget *w, gpointer client_data){
+		  theCustomPalette = (PaletteEntriesP)client_data;
+}
+
 static void
 indexed_ok_callback (GtkWidget *widget,
 		     gpointer   client_data)
@@ -380,17 +521,34 @@ indexed_ok_callback (GtkWidget *widget,
 
   if (dialog->webpal_flag) palette_type = WEB_PALETTE;
   else
-    if (dialog->monopal_flag) palette_type = MONO_PALETTE;
+    if (dialog->custompal_flag) palette_type = CUSTOM_PALETTE;
     else
-      if (dialog->makepal_flag) palette_type = MAKE_PALETTE;
+      if (dialog->monopal_flag) palette_type = MONO_PALETTE;
       else
-	palette_type = REUSE_PALETTE;
+        if (dialog->makepal_flag) palette_type = MAKE_PALETTE;
+        else
+          palette_type = REUSE_PALETTE;
   /*  Convert the image to indexed color  */
-  convert_image ((GImage *) dialog->gimage_ptr, INDEXED, dialog->num_cols, dialog->dither, palette_type);
-  gdisplays_flush ();
+  if (gimage_get_ID (dialog->gimage_ID))
+    {
+      convert_image ((GImage *) dialog->gimage_ptr, INDEXED, dialog->num_cols,
+		     dialog->dither, palette_type);
+      gdisplays_flush ();
+    }
 
   gtk_widget_destroy (dialog->shell);
   g_free (dialog);
+  dialog = NULL;
+}
+
+static gint
+indexed_delete_callback (GtkWidget *w,
+			 GdkEvent *e,
+			 gpointer client_data)
+{
+  indexed_cancel_callback (w, client_data);
+
+  return TRUE;
 }
 
 static void
@@ -402,6 +560,7 @@ indexed_cancel_callback (GtkWidget *widget,
   dialog = (IndexedDialog *) client_data;
   gtk_widget_destroy (dialog->shell);
   g_free (dialog);
+  dialog = NULL;
 }
 
 static void
@@ -413,7 +572,7 @@ indexed_num_cols_update (GtkWidget *w,
 
   str = gtk_entry_get_text (GTK_ENTRY (w));
   dialog = (IndexedDialog *) data;
-  dialog->num_cols = BOUNDS(((int) atof(str)), 1, 256);
+  dialog->num_cols = BOUNDS(((int) atof (str)), 1, 256);
 }
 
 static void
@@ -451,13 +610,17 @@ convert_image (GImage *gimage,
   QuantizeObj *quantobj;
   Layer *layer;
   Layer *floating_layer;
-  link_ptr list;
   int old_type;
+  GSList *list;
   int new_layer_type;
   int new_layer_bytes;
   int has_alpha;
   TileManager *new_tiles;
 
+  quantobj        = NULL;
+  new_layer_type  = RGBA_GIMAGE;
+  new_layer_bytes = 4;
+
   /*  Get the floating layer if one exists  */
   floating_layer = gimage_floating_sel (gimage);
 
@@ -485,27 +648,72 @@ convert_image (GImage *gimage,
 
       quantobj = initialize_median_cut (old_type, num_cols, dither ? FSDITHER : NODITHER, palette_type);
 
-
       if (palette_type == MAKE_PALETTE)
 	{
 	  if (old_type == GRAY)
 	    zero_histogram_gray (quantobj->histogram);
 	  else
 	    zero_histogram_rgb (quantobj->histogram);
+
+	  /* To begin, assume that there are fewer colours in
+	   *  the image than the user actually asked for.  In that
+	   *  case, we don't need to quantize or dither.
+	   */
+	  needs_quantize = FALSE;
+	  num_found_cols = 0;
+
 	  /*  Build the histogram  */
 	  list = gimage->layers;
 	  while (list)
 	    {
 	      layer = (Layer *) list->data;
-	      list = next_item (list);
+	      list = g_slist_next (list);
 	      if (old_type == GRAY)
 		generate_histogram_gray (quantobj->histogram, layer);
 	      else
-		generate_histogram_rgb (quantobj->histogram, layer);
+		generate_histogram_rgb (quantobj->histogram, layer, num_cols);
+	      /*
+	       * Note: generate_histogram_rgb may set needs_quantize if
+	       *  the image contains more colours than the limit specified
+	       *  by the user.
+	       */
 	    }
 	}
 
-      (* quantobj->first_pass) (quantobj);
+       if (
+         (old_type == RGB) &&
+         (!needs_quantize) &&
+         (palette_type == MAKE_PALETTE)
+         )
+
+	{
+         /* If this is an RGB image, and the user wanted a custom-built
+          *  generated palette, and this image has no more colours than
+          *  the user asked for, we don't need the first pass (quantization).
+          *
+          * There's also no point in dithering, since there's no error to
+          *  spread.  So we destroy the old quantobj and make a new one
+          *  with the remapping function set to a special LUT-based
+          *  no-dither remapper.
+          */
+
+	  quantobj->delete_func (quantobj);
+	  quantobj = initialize_median_cut (old_type, num_cols,
+					    NODESTRUCTDITHER, palette_type);
+	  /* We can skip the first pass (palette creation) */
+
+	  quantobj->actual_number_of_colors = num_found_cols;
+	  for (i = 0; i < num_found_cols; i++)
+	    {
+	      quantobj->cmap[i].red = found_cols[i][0];
+	      quantobj->cmap[i].green = found_cols[i][1];
+	      quantobj->cmap[i].blue = found_cols[i][2];
+	    }
+	}
+      else
+	{
+	  (* quantobj->first_pass) (quantobj);
+	}
 
       if (gimage->cmap)
 	g_free (gimage->cmap);
@@ -525,7 +733,7 @@ convert_image (GImage *gimage,
   while (list)
     {
       layer = (Layer *) list->data;
-      list = next_item (list);
+      list = g_slist_next (list);
 
       has_alpha = layer_has_alpha (layer);
       switch (new_type)
@@ -546,7 +754,7 @@ convert_image (GImage *gimage,
 	  break;
 	}
 
-      new_tiles = tile_manager_new (layer->width, layer->height, new_layer_bytes);
+      new_tiles = tile_manager_new (GIMP_DRAWABLE(layer)->width, GIMP_DRAWABLE(layer)->height, new_layer_bytes);
 
       switch (new_type)
 	{
@@ -566,11 +774,16 @@ convert_image (GImage *gimage,
       /*  Push the layer on the undo stack  */
       undo_push_layer_mod (gimage, layer);
 
-      layer->tiles = new_tiles;
-      layer->bytes = new_layer_bytes;
-      layer->type = new_layer_type;
+      GIMP_DRAWABLE(layer)->tiles = new_tiles;
+      GIMP_DRAWABLE(layer)->bytes = new_layer_bytes;
+      GIMP_DRAWABLE(layer)->type = new_layer_type;
+      GIMP_DRAWABLE(layer)->has_alpha = TYPE_HAS_ALPHA(new_layer_type);
     }
 
+  /*  Delete the quantizer object, if there is one */
+  if (quantobj) 
+    quantobj->delete_func (quantobj);
+
   /*  Make sure the projection is up to date  */
   gimage_projection_realloc (gimage);
 
@@ -604,8 +817,8 @@ rgb_converter (Layer       *layer,
   void *pr;
 
   has_alpha = layer_has_alpha (layer);
-  pixel_region_init (&srcPR, layer->tiles, 0, 0, layer->width, layer->height, FALSE);
-  pixel_region_init (&destPR, new_tiles, 0, 0, layer->width, layer->height, TRUE);
+  pixel_region_init (&srcPR, GIMP_DRAWABLE(layer)->tiles, 0, 0, GIMP_DRAWABLE(layer)->width, GIMP_DRAWABLE(layer)->height, FALSE);
+  pixel_region_init (&destPR, new_tiles, 0, 0, GIMP_DRAWABLE(layer)->width, GIMP_DRAWABLE(layer)->height, TRUE);
 
   for (pr = pixel_regions_register (2, &srcPR, &destPR); pr != NULL; pr = pixel_regions_process (pr))
     {
@@ -636,7 +849,7 @@ rgb_converter (Layer       *layer,
 	    }
 	  break;
 	case INDEXED:
-	  cmap = drawable_cmap (layer->ID);
+	  cmap = drawable_cmap (GIMP_DRAWABLE(layer));
 	  for (row = 0; row < srcPR.h; row++)
 	    {
 	      s = src;
@@ -678,8 +891,8 @@ grayscale_converter (Layer       *layer,
   void *pr;
 
   has_alpha = layer_has_alpha (layer);
-  pixel_region_init (&srcPR, layer->tiles, 0, 0, layer->width, layer->height, FALSE);
-  pixel_region_init (&destPR, new_tiles, 0, 0, layer->width, layer->height, TRUE);
+  pixel_region_init (&srcPR, GIMP_DRAWABLE(layer)->tiles, 0, 0, GIMP_DRAWABLE(layer)->width, GIMP_DRAWABLE(layer)->height, FALSE);
+  pixel_region_init (&destPR, new_tiles, 0, 0, GIMP_DRAWABLE(layer)->width, GIMP_DRAWABLE(layer)->height, TRUE);
 
   for (pr = pixel_regions_register (2, &srcPR, &destPR); pr != NULL; pr = pixel_regions_process (pr))
     {
@@ -707,7 +920,7 @@ grayscale_converter (Layer       *layer,
 	    }
 	  break;
 	case INDEXED:
-	  cmap = drawable_cmap (layer->ID);
+	  cmap = drawable_cmap (GIMP_DRAWABLE(layer));
 	  for (row = 0; row < srcPR.h; row++)
 	    {
 	      s = src;
@@ -766,8 +979,11 @@ generate_histogram_gray (Histogram  histogram,
   unsigned char *data;
   int size;
   void *pr;
+  gboolean has_alpha;
 
-  pixel_region_init (&srcPR, layer->tiles, 0, 0, layer->width, layer->height, FALSE);
+  has_alpha = (gboolean) layer_has_alpha(layer);
+
+  pixel_region_init (&srcPR, GIMP_DRAWABLE(layer)->tiles, 0, 0, GIMP_DRAWABLE(layer)->width, GIMP_DRAWABLE(layer)->height, FALSE);
   for (pr = pixel_regions_register (1, &srcPR);
        pr != NULL;
        pr = pixel_regions_process (pr))
@@ -785,30 +1001,107 @@ generate_histogram_gray (Histogram  histogram,
 
 static void
 generate_histogram_rgb (Histogram  histogram,
-			Layer     *layer)
+			Layer     *layer,
+			int        col_limit)
 {
   PixelRegion srcPR;
   unsigned char *data;
   int size;
   void *pr;
   ColorFreq *col;
+  gboolean has_alpha;
+  int nfc_iter;
 
-  pixel_region_init (&srcPR, layer->tiles, 0, 0, layer->width, layer->height, FALSE);
+  has_alpha = (gboolean) layer_has_alpha(layer);
+
+/*  g_print ("col_limit = %d, nfc = %d\n", col_limit, num_found_cols);*/
+
+  pixel_region_init (&srcPR, GIMP_DRAWABLE(layer)->tiles, 0, 0,
+		     GIMP_DRAWABLE(layer)->width,
+		     GIMP_DRAWABLE(layer)->height,
+		     FALSE);
   for (pr = pixel_regions_register (1, &srcPR);
        pr != NULL;
        pr = pixel_regions_process (pr))
     {
       data = srcPR.data;
       size = srcPR.w * srcPR.h;
-      while (size--)
+
+      if (needs_quantize)
 	{
-	  col = & histogram[(data[RED_PIX] >> R_SHIFT) * MR +
-			   (data[GREEN_PIX] >> G_SHIFT) * MG +
-			   (data[BLUE_PIX] >> B_SHIFT)];
-	  (*col)++;
-	  data += srcPR.bytes;
+	  while (size--)
+	    {
+	      if ((has_alpha && (data[ALPHA_PIX]&128)) || (!has_alpha))
+		{
+		  col = & histogram[(data[RED_PIX] >> R_SHIFT) * MR +
+				    (data[GREEN_PIX] >> G_SHIFT) * MG +
+				    (data[BLUE_PIX] >> B_SHIFT)];
+		  (*col)++;
+		}
+	      data += srcPR.bytes;
+	    }
+	}
+      else
+	{
+	  while (size--)
+	    {
+	      if ((has_alpha && (data[ALPHA_PIX]&128)) || (!has_alpha))
+		{
+		  col = & histogram[(data[RED_PIX] >> R_SHIFT) * MR +
+				    (data[GREEN_PIX] >> G_SHIFT) * MG +
+				    (data[BLUE_PIX] >> B_SHIFT)];
+		  (*col)++;
+
+		  if (!needs_quantize)
+		    {
+		      for (nfc_iter = 0;
+			   nfc_iter < num_found_cols;
+			   nfc_iter++)
+			{
+			  if (
+			      (data[RED_PIX] == found_cols[nfc_iter][0])
+			      &&
+			      (data[GREEN_PIX] == found_cols[nfc_iter][1])
+			      &&
+			      (data[BLUE_PIX] == found_cols[nfc_iter][2])
+			      )
+			    goto already_found;
+			}
+			  
+		      /* Colour was not in the table of
+		       * existing colours
+		       */
+		      
+		      num_found_cols++;
+		      
+		      if (num_found_cols > col_limit)
+			{
+			  /* There are more colours in the image
+			   *  than were allowed.  We switch to plain
+			   *  histogram calculation with a view to
+			   *  quantizing at a later stage.
+			   */
+			  needs_quantize = TRUE;
+/*			  g_print ("\nmax colours exceeded - needs quantize.\n");*/
+			  goto already_found;
+			}
+		      else
+			{
+			  /* Remember the new colour we just found.
+			   */
+			  found_cols[num_found_cols-1][0] = data[RED_PIX];
+			  found_cols[num_found_cols-1][1] = data[GREEN_PIX];
+			  found_cols[num_found_cols-1][2] = data[BLUE_PIX];
+			}
+		    }
+		}
+	    already_found:
+	      data += srcPR.bytes;	      
+	    }
 	}
     }
+
+/*  g_print ("O: col_limit = %d, nfc = %d\n", col_limit, num_found_cols);*/
 }
 
 
@@ -1191,9 +1484,21 @@ compute_color_gray (QuantizeObj *quantobj,
 	}
     }
 
-  quantobj->cmap[icolor].red = (gtotal + (total >> 1)) / total;
-  quantobj->cmap[icolor].green = quantobj->cmap[icolor].red;
-  quantobj->cmap[icolor].blue = quantobj->cmap[icolor].red;
+  if (total != 0)
+    {
+      quantobj->cmap[icolor].red = (gtotal + (total >> 1)) / total;
+      quantobj->cmap[icolor].green = quantobj->cmap[icolor].red;
+      quantobj->cmap[icolor].blue = quantobj->cmap[icolor].red;
+    }
+   else /* The only situation where total==0 is if the image was null or
+	*  all-transparent.  In that case we just put a dummy value in
+	*  the colourmap.
+	*/
+    {
+      quantobj->cmap[icolor].red =
+	quantobj->cmap[icolor].green =
+	quantobj->cmap[icolor].blue = 0;
+    }
 }
 
 static void
@@ -1236,9 +1541,21 @@ compute_color_rgb (QuantizeObj *quantobj,
 	  }
       }
 
-  quantobj->cmap[icolor].red = (Rtotal + (total>>1)) / total;
-  quantobj->cmap[icolor].green = (Gtotal + (total>>1)) / total;
-  quantobj->cmap[icolor].blue = (Btotal + (total>>1)) / total;
+  if (total != 0)
+    {
+      quantobj->cmap[icolor].red = (Rtotal + (total>>1)) / total;
+      quantobj->cmap[icolor].green = (Gtotal + (total>>1)) / total;
+      quantobj->cmap[icolor].blue = (Btotal + (total>>1)) / total;
+    }
+  else /* The only situation where total==0 is if the image was null or
+	*  all-transparent.  In that case we just put a dummy value in
+	*  the colourmap.
+	*/
+    {
+      quantobj->cmap[icolor].red =
+	quantobj->cmap[icolor].green =
+	quantobj->cmap[icolor].blue = 0;
+    }
 }
 
 
@@ -1731,6 +2048,28 @@ webpal_pass1 (QuantizeObj *quantobj)
       quantobj->cmap[i].blue = webpal[i*3 +2];
     }
 }
+static void
+custompal_pass1 (QuantizeObj *quantobj)
+{
+  int i;
+  GSList *list;
+  PaletteEntryP entry;
+
+  /*  fprintf(stderr, "custompal_pass1: using (theCustomPalette %s) from (file %s)\n",
+			 theCustomPalette->name, theCustomPalette->filename);*/
+
+  for (i=0,list=theCustomPalette->colors;
+       list;
+       i++,list=g_slist_next(list))
+    {
+      entry=(PaletteEntryP)list->data;
+      quantobj->cmap[i].red = entry->color[0];
+      quantobj->cmap[i].green = entry->color[1];
+      quantobj->cmap[i].blue = entry->color[2];
+    }
+  quantobj -> actual_number_of_colors = i;
+}
+
 /*
  * Map some rows of pixels to the output colormapped representation.
  */
@@ -1752,8 +2091,8 @@ median_cut_pass2_no_dither_gray (QuantizeObj *quantobj,
   zero_histogram_gray (histogram);
 
   has_alpha = layer_has_alpha (layer);
-  pixel_region_init (&srcPR, layer->tiles, 0, 0, layer->width, layer->height, FALSE);
-  pixel_region_init (&destPR, new_tiles, 0, 0, layer->width, layer->height, TRUE);
+  pixel_region_init (&srcPR, GIMP_DRAWABLE(layer)->tiles, 0, 0, GIMP_DRAWABLE(layer)->width, GIMP_DRAWABLE(layer)->height, FALSE);
+  pixel_region_init (&destPR, new_tiles, 0, 0, GIMP_DRAWABLE(layer)->width, GIMP_DRAWABLE(layer)->height, TRUE);
   for (pr = pixel_regions_register (2, &srcPR, &destPR); pr != NULL; pr = pixel_regions_process (pr))
     {
       src = srcPR.data;
@@ -1802,14 +2141,14 @@ median_cut_pass2_no_dither_rgb (QuantizeObj *quantobj,
   /*  In the case of web/mono palettes, we actually force
    *   grayscale drawables through the rgb pass2 functions
    */
-  if (drawable_gray (layer->ID))
+  if (drawable_gray (GIMP_DRAWABLE(layer)))
     red_pix = green_pix = blue_pix = GRAY_PIX;
 
   zero_histogram_rgb (histogram);
 
   has_alpha = layer_has_alpha (layer);
-  pixel_region_init (&srcPR, layer->tiles, 0, 0, layer->width, layer->height, FALSE);
-  pixel_region_init (&destPR, new_tiles, 0, 0, layer->width, layer->height, TRUE);
+  pixel_region_init (&srcPR, GIMP_DRAWABLE(layer)->tiles, 0, 0, GIMP_DRAWABLE(layer)->width, GIMP_DRAWABLE(layer)->height, FALSE);
+  pixel_region_init (&destPR, new_tiles, 0, 0, GIMP_DRAWABLE(layer)->width, GIMP_DRAWABLE(layer)->height, TRUE);
   for (pr = pixel_regions_register (2, &srcPR, &destPR); pr != NULL; pr = pixel_regions_process (pr))
     {
       src = srcPR.data;
@@ -1840,6 +2179,90 @@ median_cut_pass2_no_dither_rgb (QuantizeObj *quantobj,
     }
 }
 
+static void
+median_cut_pass2_nodestruct_dither_rgb (QuantizeObj *quantobj,
+					Layer       *layer,
+					TileManager *new_tiles)
+{
+  PixelRegion srcPR, destPR;
+  unsigned char *src, *dest;
+  int row, col;
+  int has_alpha;
+  void *pr;
+  int red_pix = RED_PIX;
+  int green_pix = GREEN_PIX;
+  int blue_pix = BLUE_PIX;
+  int alpha_pix = ALPHA_PIX;
+  int i;
+  int lastindex = 0;
+  int lastred = -1;
+  int lastgreen = -1;
+  int lastblue = -1;
+
+
+  has_alpha = layer_has_alpha (layer);
+  pixel_region_init (&srcPR, GIMP_DRAWABLE(layer)->tiles, 0, 0, GIMP_DRAWABLE(layer)->width, GIMP_DRAWABLE(layer)->height, FALSE);
+  pixel_region_init (&destPR, new_tiles, 0, 0, GIMP_DRAWABLE(layer)->width, GIMP_DRAWABLE(layer)->height, TRUE);
+  for (pr = pixel_regions_register (2, &srcPR, &destPR); pr != NULL; pr = pixel_regions_process (pr))
+    {
+      src = srcPR.data;
+      dest = destPR.data;
+      for (row = 0; row < srcPR.h; row++)
+	{
+	  for (col = 0; col < srcPR.w; col++)
+	    {
+	      if ((has_alpha && (src[alpha_pix]>127))
+		  || !has_alpha)
+		{
+		  if ((lastred == src[red_pix]) &&
+		      (lastgreen == src[green_pix]) &&
+		      (lastblue == src[blue_pix]))
+		    {
+		      /*  same pixel colour as last time  */
+		      dest[INDEXED_PIX] = lastindex;
+		      if (has_alpha)
+			dest[ALPHA_I_PIX] = 255;
+		    }
+		  else
+		    {
+		      for (i = 0 ;
+			   i < quantobj->actual_number_of_colors;
+			   i++)
+			{
+			  if (
+			      (quantobj->cmap[i].red == src[red_pix]) &&
+			      (quantobj->cmap[i].green == src[green_pix]) &&
+			      (quantobj->cmap[i].blue == src[blue_pix])
+			      )
+			  {
+			    lastred = src[red_pix];
+			    lastgreen = src[green_pix];
+			    lastblue = src[blue_pix];
+			    lastindex = i;
+			    goto got_colour;
+			  }
+			}
+		      g_error ("Non-existant colour was expected to "
+			       "be in non-destructive colourmap.");
+		    got_colour:
+		      dest[INDEXED_PIX] = lastindex;
+		      if (has_alpha)
+			dest[ALPHA_I_PIX] = 255;
+		    }
+		}
+	      else
+		{ /*  have alpha, and transparent  */
+		  dest[ALPHA_I_PIX] = 0;
+		}
+
+	      src += srcPR.bytes;
+	      dest += destPR.bytes;
+	    }
+	}
+    }
+}
+
+
 /*
  * Initialize the error-limiting transfer function (lookup table).
  * The raw F-S error computation can potentially compute error values of up to
@@ -1933,12 +2356,12 @@ median_cut_pass2_fs_dither_gray (QuantizeObj *quantobj,
   zero_histogram_gray (histogram);
 
   has_alpha = layer_has_alpha (layer);
-  pixel_region_init (&srcPR, layer->tiles, 0, 0, layer->width, layer->height, FALSE);
-  pixel_region_init (&destPR, new_tiles, 0, 0, layer->width, layer->height, TRUE);
-  src_bytes = layer->bytes;
+  pixel_region_init (&srcPR, GIMP_DRAWABLE(layer)->tiles, 0, 0, GIMP_DRAWABLE(layer)->width, GIMP_DRAWABLE(layer)->height, FALSE);
+  pixel_region_init (&destPR, new_tiles, 0, 0, GIMP_DRAWABLE(layer)->width, GIMP_DRAWABLE(layer)->height, TRUE);
+  src_bytes = GIMP_DRAWABLE(layer)->bytes;
   dest_bytes = new_tiles->levels[0].bpp;
-  width = layer->width;
-  height = layer->height;
+  width = GIMP_DRAWABLE(layer)->width;
+  height = GIMP_DRAWABLE(layer)->height;
 
   error_limiter = init_error_limit ();
   range_limiter = range_array + 256;
@@ -2083,18 +2506,18 @@ median_cut_pass2_fs_dither_rgb (QuantizeObj *quantobj,
   /*  In the case of web/mono palettes, we actually force
    *   grayscale drawables through the rgb pass2 functions
    */
-  if (drawable_gray (layer->ID))
+  if (drawable_gray (GIMP_DRAWABLE(layer)))
     red_pix = green_pix = blue_pix = GRAY_PIX;
 
   zero_histogram_rgb (histogram);
 
   has_alpha = layer_has_alpha (layer);
-  pixel_region_init (&srcPR, layer->tiles, 0, 0, layer->width, layer->height, FALSE);
-  pixel_region_init (&destPR, new_tiles, 0, 0, layer->width, layer->height, TRUE);
-  src_bytes = layer->bytes;
+  pixel_region_init (&srcPR, GIMP_DRAWABLE(layer)->tiles, 0, 0, GIMP_DRAWABLE(layer)->width, GIMP_DRAWABLE(layer)->height, FALSE);
+  pixel_region_init (&destPR, new_tiles, 0, 0, GIMP_DRAWABLE(layer)->width, GIMP_DRAWABLE(layer)->height, TRUE);
+  src_bytes = GIMP_DRAWABLE(layer)->bytes;
   dest_bytes = new_tiles->levels[0].bpp;
-  width = layer->width;
-  height = layer->height;
+  width = GIMP_DRAWABLE(layer)->width;
+  height = GIMP_DRAWABLE(layer)->height;
 
   error_limiter = init_error_limit ();
   range_limiter = range_array + 256;
@@ -2181,6 +2604,10 @@ median_cut_pass2_fs_dither_rgb (QuantizeObj *quantobj,
 	  if (has_alpha)
 	    dest[ALPHA_I_PIX] = (src[alpha_pix] > 127) ? 255 : 0;
 
+          /* Abortive initial attempt at alpha-dithering :) */
+	  /*	  if (has_alpha)
+	    dest[ALPHA_I_PIX] = (src[alpha_pix] <= (random()&255)) ? 0 : 255;*/
+
 	  color = &quantobj->cmap[index];
 	  re = r - color->red;
 	  ge = g - color->green;
@@ -2299,12 +2726,16 @@ initialize_median_cut (int type,
 	case WEB_PALETTE:
 	  quantobj->first_pass = webpal_pass1;
 	  break;
+	case CUSTOM_PALETTE:
+	  quantobj->first_pass = custompal_pass1;
+	  needs_quantize=TRUE;
+	  break;
 	case MONO_PALETTE:
 	default:
 	  quantobj->first_pass = monopal_pass1;
 	}
       if (palette_type == WEB_PALETTE ||
-	  palette_type == MONO_PALETTE)
+	  palette_type == MONO_PALETTE || palette_type == CUSTOM_PALETTE)
 	switch (dither_type)
 	  {
 	  case NODITHER:
@@ -2333,6 +2764,11 @@ initialize_median_cut (int type,
 	  break;
 	case WEB_PALETTE:
 	  quantobj->first_pass = webpal_pass1;
+	  needs_quantize=TRUE;
+	  break;
+	case CUSTOM_PALETTE:
+	  quantobj->first_pass = custompal_pass1;
+	  needs_quantize=TRUE;
 	  break;
 	case MONO_PALETTE:
 	default:
@@ -2346,6 +2782,9 @@ initialize_median_cut (int type,
 	case FSDITHER:
 	  quantobj->second_pass = median_cut_pass2_fs_dither_rgb;
 	  break;
+	case NODESTRUCTDITHER:
+	  quantobj->second_pass = median_cut_pass2_nodestruct_dither_rgb;
+	  break;
 	}
       break;
     }
@@ -2529,7 +2968,7 @@ convert_indexed_invoker (Argument *args)
     {
       int_value = args[0].value.pdb_int;
       if (! (gimage = gimage_get_ID (int_value)))
-	success = FALSE;
+        success = FALSE;
     }
   /*  make sure the drawable is not INDEXED color  */
   if (success)
@@ -2540,7 +2979,7 @@ convert_indexed_invoker (Argument *args)
     {
       num_cols = args[2].value.pdb_int;
       if (num_cols < 1 || num_cols > MAXNUMCOLORS)
-	success = FALSE;
+        success = FALSE;
     }
 
   if (success)
@@ -2548,3 +2987,124 @@ convert_indexed_invoker (Argument *args)
 
   return procedural_db_return_args (&convert_indexed_proc, success);
 }
+
+
+
+/*  The convert-indexed-palette procedure definition  */
+ProcArg convert_indexed_palette_args[] =
+{
+  { PDB_IMAGE,
+    "image",
+    "the image"
+  },
+  { PDB_INT32,
+    "dither",
+    "Floyd-Steinberg dithering"
+  },
+  { PDB_INT32,
+    "palette_type",
+    "The type of palette to use, (0 optimal) (1 reuse) (2 WWW) (3 Mono) (4 Custom)"
+  },
+  { PDB_INT32,
+    "num_cols",
+    "the number of colors to quantize to, ignored unless (palette_type == 0)"
+  },
+  { PDB_STRING,
+    "palette",
+    "The name of the custom palette to use, ignored unless (palette_type == 4)"
+  }
+};
+
+ProcRecord convert_indexed_palette_proc =
+{
+  "gimp_convert_indexed_palette",
+  "Convert specified image to indexed color",
+  "This procedure converts the specified image to indexed color.  This process requires an image of type GRAY or RGB.  The `palette_type' specifies what kind of palette to use, A type of `0' means to use an optimal palette of `num_cols' generated from the colors in the image.  A type of `1' means to re-use the previous palette.  A type of `2' means to use the WWW-optimized palette.  Type `3' means to use only black and white colors.  A type of `4' means to use a palette from the gimp palettes directories.",
+  "Spencer Kimball & Peter Mattis",
+  "Spencer Kimball & Peter Mattis",
+  "1995-1996",
+  PDB_INTERNAL,
+
+  /*  Input arguments  */
+  5,
+  convert_indexed_palette_args,
+
+  /*  Output arguments  */
+  0,
+  NULL,
+
+  /*  Exec method  */
+  { { convert_indexed_palette_invoker } },
+};
+
+
+static Argument *
+convert_indexed_palette_invoker (Argument *args)
+{
+  int success = TRUE;
+  int int_value;
+  GImage *gimage;
+  int dither;
+  int num_cols=0;
+  int palette_type;
+  char *palette_name;
+
+  /*  the gimage  */
+  if (success)
+    {
+      int_value = args[0].value.pdb_int;
+      if (! (gimage = gimage_get_ID (int_value)))
+	success = FALSE;
+    }
+  /*  make sure the drawable is not INDEXED color  */
+  if (success)
+    success = (gimage_base_type (gimage) != INDEXED);
+  if (success)
+    dither = (args[1].value.pdb_int) ? TRUE : FALSE;
+  if (success)
+    {
+      PaletteEntriesP entries, the_palette = NULL;
+      GSList *list;
+
+		palette_type = args[2].value.pdb_int;
+		switch(palette_type) {
+		  case MAKE_PALETTE:
+          num_cols = args[3].value.pdb_int;
+          if (num_cols < 1 || num_cols > MAXNUMCOLORS)
+            success = FALSE;
+		  break;
+		  case REUSE_PALETTE:
+		  break;
+		  case WEB_PALETTE:
+		  break;
+		  case MONO_PALETTE:
+		  break;
+		  case CUSTOM_PALETTE:
+          palette_name = args[4].value.pdb_pointer;
+ /*         fprintf(stderr, "looking for palette `%s'\n", palette_name); */
+          if (!palette_entries_list) palette_init_palettes(FALSE);
+		    for(list = palette_entries_list;
+              list;
+              list = g_slist_next(list)) {
+                entries = (PaletteEntriesP) list->data;
+                if (strcmp(palette_name, entries->name)==0) {
+	/*					fprintf(stderr, "found it!\n"); */
+                  the_palette = entries;
+                  break;
+					 }
+			 }
+          if (the_palette == NULL) {
+	/*			fprintf(stderr, "didn't find it\n"); */
+            success = FALSE;
+			 }
+          else
+            theCustomPalette = the_palette;
+		  break;
+		  default:
+          success = FALSE;
+		}
+	 }
+    if (success)
+       convert_image ((void *) gimage, INDEXED, num_cols, dither, palette_type);
+  return procedural_db_return_args (&convert_indexed_proc, success);
+}
