@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 #include <stdlib.h>
 #include <string.h>
@@ -25,7 +25,7 @@
 /*  static functions  */
 static InfoField * info_field_new (InfoDialog *, char *, char *);
 static void        update_field (InfoField *);
-
+static gint        info_dialog_delete_callback (GtkWidget *, GdkEvent *, gpointer);
 
 static InfoField *
 info_field_new (InfoDialog *idialog,
@@ -83,9 +83,14 @@ info_dialog_new (char *title)
   idialog->field_list = NULL;
 
   shell = gtk_dialog_new ();
+  gtk_window_set_wmclass (GTK_WINDOW (shell), "info_dialog", "Gimp");
   gtk_window_set_title (GTK_WINDOW (shell), title);
   gtk_widget_set_uposition (shell, info_x, info_y);
 
+  gtk_signal_connect (GTK_OBJECT (shell), "delete_event",
+		      GTK_SIGNAL_FUNC (info_dialog_delete_callback),
+		      idialog);
+
   vbox = gtk_vbox_new (FALSE, 1);
   gtk_container_border_width (GTK_CONTAINER (vbox), 1);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (shell)->vbox), vbox, TRUE, TRUE, 0);
@@ -117,7 +122,7 @@ info_dialog_new (char *title)
 void
 info_dialog_free (InfoDialog *idialog)
 {
-  link_ptr list;
+  GSList *list;
 
   if (!idialog)
     return;
@@ -128,11 +133,11 @@ info_dialog_free (InfoDialog *idialog)
   while (list)
     {
       g_free (list->data);
-      list = next_item (list);
+      list = g_slist_next (list);
     }
 
   /*  Free the actual field linked list  */
-  free_list (idialog->field_list);
+  g_slist_free (idialog->field_list);
 
   /*  Destroy the associated widgets  */
   gtk_widget_destroy (idialog->shell);
@@ -152,7 +157,7 @@ info_dialog_add_field (InfoDialog *idialog,
     return;
 
   new_field = info_field_new (idialog, title, text_ptr);
-  idialog->field_list = add_to_list (idialog->field_list, (void *) new_field);
+  idialog->field_list = g_slist_prepend (idialog->field_list, (void *) new_field);
 }
 
 void
@@ -170,7 +175,7 @@ info_dialog_popdown (InfoDialog *idialog)
 {
   if (!idialog)
     return;
-
+  
   if (GTK_WIDGET_VISIBLE (idialog->shell))
     gtk_widget_hide (idialog->shell);
 }
@@ -178,7 +183,7 @@ info_dialog_popdown (InfoDialog *idialog)
 void
 info_dialog_update (InfoDialog *idialog)
 {
-  link_ptr list;
+  GSList *list;
 
   if (!idialog)
     return;
@@ -188,6 +193,21 @@ info_dialog_update (InfoDialog *idialog)
   while (list)
     {
       update_field ((InfoField *) list->data);
-      list = next_item (list);
+      list = g_slist_next (list);
     }
 }
+
+static gint
+info_dialog_delete_callback (GtkWidget *w,
+			     GdkEvent *e,
+			     gpointer client_data)
+{
+  info_dialog_popdown ((InfoDialog *) client_data);
+
+  return TRUE;
+}
+
+
+
+
+
