@@ -13,16 +13,21 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 #ifndef __PAINTBRUSH_H__
 #define __PAINTBRUSH_H__
 
 #include "tools.h"
+#include "paint_core.h"
+#include "procedural_db.h"
 
-void          paintbrush_dialog      (void);
-void *        paintbrush_paint_func  (Tool *, int);
+void *        paintbrush_paint_func  (PaintCore *, GimpDrawable *, int);
 Tool *        tools_new_paintbrush   (void);
 void          tools_free_paintbrush  (Tool *);
 
+/*  Procedure definition and marshalling function  */
+extern ProcRecord paintbrush_proc;
+extern ProcRecord paintbrush_extended_proc;
+
 #endif  /*  __PAINTBRUSH_H__  */
