@@ -1,17 +1,42 @@
+/* LIBGIMP - The GIMP Library                                                   
+ * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball                
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.             
+ *                                                                              
+ * This library is distributed in the hope that it will be useful,              
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of               
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU            
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */                                                                             
 #ifndef __GIMP_H__
 #define __GIMP_H__
 
 
-#include <glib/glib.h>
+#include <glib.h>
 #include <libgimp/gimpenums.h>
 
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 
 typedef struct _GPlugInInfo  GPlugInInfo;
 typedef struct _GTile        GTile;
 typedef struct _GDrawable    GDrawable;
 typedef struct _GPixelRgn    GPixelRgn;
 typedef struct _GParamDef    GParamDef;
+typedef struct _GParamColor  GParamColor;
+typedef struct _GParamRegion GParamRegion;
+typedef union  _GParamData   GParamData;
 typedef struct _GParam       GParam;
 typedef void   (* GRunProc) (char    *name,
 			     int      nparams,
@@ -89,42 +114,50 @@ struct _GParamDef
   char *description;
 };
 
+struct _GParamColor
+{
+  guint8 red;
+  guint8 green;
+  guint8 blue;
+};
+
+struct _GParamRegion
+{
+  gint32 x;
+  gint32 y;
+  gint32 width;
+  gint32 height;
+};
+
+union _GParamData
+{
+  gint32 d_int32;
+  gint16 d_int16;
+  gint8 d_int8;
+  gdouble d_float;
+  gchar *d_string;
+  gint32 *d_int32array;
+  gint16 *d_int16array;
+  gint8 *d_int8array;
+  gdouble *d_floatarray;
+  gchar **d_stringarray;
+  GParamColor d_color;
+  GParamRegion d_region;
+  gint32 d_display;
+  gint32 d_image;
+  gint32 d_layer;
+  gint32 d_channel;
+  gint32 d_drawable;
+  gint32 d_selection;
+  gint32 d_boundary;
+  gint32 d_path;
+  gint32 d_status;
+};
+
 struct _GParam
 {
   GParamType type;
-
-  union {
-    gint32 d_int32;
-    gint16 d_int16;
-    gint8 d_int8;
-    gdouble d_float;
-    gchar *d_string;
-    gint32 *d_int32array;
-    gint16 *d_int16array;
-    gint8 *d_int8array;
-    gdouble *d_floatarray;
-    gchar **d_stringarray;
-    struct {
-      guint8 red;
-      guint8 green;
-      guint8 blue;
-    } d_color;
-    struct {
-      gint32 x;
-      gint32 y;
-      gint32 width;
-      gint32 height;
-    } d_region;
-    gint32 d_display;
-    gint32 d_image;
-    gint32 d_layer;
-    gint32 d_channel;
-    gint32 d_drawable;
-    gint32 d_selection;
-    gint32 d_boundary;
-    gint32 d_path;
-    gint32 d_status;
-  } data;
+  GParamData data;
 };
 
 
@@ -173,6 +206,12 @@ void gimp_progress_init (char *message);
 void gimp_progress_update (gdouble percentage);
 
 
+/* Pops up a dialog box with "message". Useful for status and
+ * error reports. If "message" is NULL, do nothing.
+ */
+void gimp_message (char *message);
+
+
 /* Query the gimp application's procedural database.
  *  The arguments are regular expressions which select
  *  which procedure names will be returned in 'proc_names'.
@@ -647,4 +686,9 @@ gdouble* gimp_gradients_sample_custom  (gint     num_samples,
 					gdouble *positions);
 
 
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
+
 #endif /* __GIMP_H__ */
